/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_file.h"
#ifdef ENABLE_CAPTURE_ANIMATION
#include "cam_animation.h"
#include "camera_utils.h"

static void __cam_app_shutter_animation_finished(void *data, Evas_Object *obj,
						 const char *emission,
						 const char *source)
{
	cam_debug(LOG_MM, " __cam_app_shutter_animation_finished \n\n ");
	struct appdata *ad = (struct appdata *)data;
//	edje_object_signal_emit(_EDJ(ad->shutter_screen), "bright", "");
	DEL_EVAS_OBJECT(ad->shutter_screen);
	DEL_EVAS_OBJECT(ad->rect_image);

	ad->is_capture_animation_processing = FALSE;
}

static void __cam_animation_get_preview_coordinate_by_direction(int *preview_offset_x,
										int *preview_offset_y,
										int *preview_w,
										int *preview_h,
										void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		*preview_offset_x = ad->preview_offset_x;
		*preview_offset_y = ad->preview_offset_y;
		*preview_w = ad->preview_w;
		*preview_h = ad->preview_h;
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		*preview_offset_x = ad->preview_offset_y;
		*preview_offset_y = ad->preview_offset_x;
		*preview_w = ad->preview_h;
		*preview_h = ad->preview_w;
		break;
	default:
		DEBUG_TRACE("reached unable reached codes. error");
	}
}

void cam_animation_create_rect_image(void *data)
{
	struct appdata *ad = data;
	Evas_Coord witdh = 0, height = 0;

	char *group_name = "shutter_rect";

	DEL_EVAS_OBJECT(ad->rect_image);

	ad->rect_image = cam_app_load_edj(ad->ug_base, CAM_MAIN_LAYOUT_EDJ_NAME, group_name);
	cam_retm_if(ad->rect_image == NULL, "rect_image load failed");

	ecore_x_window_size_get(ad->main_xid, &witdh, &height);
	evas_object_resize(ad->rect_image, witdh, height);
	edje_object_signal_callback_add(_EDJ(ad->rect_image),
					"shutter_rect,finish", "*",
					__cam_app_shutter_animation_finished,
					ad);
	return;

}

Eina_Bool cam_start_capture_animation(void *data)
{
	cam_debug(LOG_MM, " cam_start_capture_animation \n\n ");

	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;

	Evas_Coord x, y, w, h;
	x = y = w = h = 0;

	int window_width, window_height;
	int pre_width, pre_height;
	int pre_x, pre_y;

	cam_retvm_if(ad == NULL, ECORE_CALLBACK_CANCEL, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, ECORE_CALLBACK_CANCEL, "camapp_handle is NULL");

	window_width = ad->win_width;
	window_height = ad->win_height;
	pre_width = ad->preview_w;
	pre_height = ad->preview_h;

	__cam_animation_get_preview_coordinate_by_direction(&pre_x, &pre_y, &pre_width, &pre_height, ad);
	if (ad->preview_w <= ad->win_width) {
		ad->shutter_src_rect.x =  pre_x;
		ad->shutter_src_rect.width = pre_width;
	} else {
		ad->shutter_src_rect.x = 0;
		ad->shutter_src_rect.width = pre_width;
	}

	if (ad->preview_h <= ad->win_height) {
		ad->shutter_src_rect.y = pre_y;
		ad->shutter_src_rect.height = pre_height;
	} else {
		ad->shutter_src_rect.y = 0;
		ad->shutter_src_rect.height = pre_height;
	}

	ad->shutter_des_rect.x = ad->shutter_src_rect.x;
	ad->shutter_des_rect.y = y ;
	ad->shutter_des_rect.width = w;
	ad->shutter_des_rect.height = h;

	evas_object_stack_above(ad->rect_image, ad->shutter_screen);
	evas_object_resize(ad->rect_image, ad->shutter_src_rect.width, ad->shutter_src_rect.height);
	evas_object_move(ad->rect_image, ad->shutter_src_rect.x, ad->shutter_src_rect.y);
	evas_object_show(ad->rect_image);
	edje_object_signal_emit(_EDJ(ad->rect_image), "shutter_rect,start", "prog");

	return EINA_TRUE;
}

gboolean cam_app_create_start_animation(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	ad->is_capture_animation_processing = TRUE;

	cam_app_create_shutter_screen(ad);
	cam_animation_create_rect_image(ad);
	cam_start_capture_animation(ad);

	return FALSE;
}

gboolean cam_app_create_screennail_and_start_animation(void *user_data)
{
	struct appdata *ad = (struct appdata *)user_data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
#if 1/*note: using pipe handle*/
	cam_utils_request_main_pipe_handler(ad, NULL, CAM_MAIN_PIPE_OP_TYPE_SHUTTER_ANIMATION);
#else
	cam_app_create_start_animation(ad);
#endif
	return TRUE;
}

gboolean cam_app_create_shutter_screen(void *data)
{
	struct appdata *ad = data;
	Evas_Coord witdh = 0, height = 0;

	char *group_name = "shutter_image";

	DEL_EVAS_OBJECT(ad->shutter_screen);

	ad->shutter_screen = cam_app_load_edj(ad->ug_base, CAM_MAIN_LAYOUT_EDJ_NAME, group_name);
	if (ad->shutter_screen == NULL) {
		cam_critical(LOG_UI, "shutter_screen load failed ");
		return FALSE;
	}

	ecore_x_window_size_get(ad->main_xid, &witdh, &height);
	evas_object_resize(ad->shutter_screen, witdh, height);

	/*evas_object_lower(ad->shutter_screen);*/
	evas_object_show(ad->shutter_screen);
	evas_object_show(ad->win_main);

	/*edje_object_signal_emit(_EDJ(ad->shutter_screen), "start_animation", "");*/

	edje_object_signal_callback_add(_EDJ(ad->shutter_screen),
					"animation_finish", "*",
					__cam_app_shutter_animation_finished,
					ad);
	return TRUE;
}
#endif
