/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_continuous_shot.h"
#include "cam_ta.h"
#include "cam_error.h"
#include "cam_mm.h"
#include "cam_sound.h"
#include "cam_file.h"
#include "camera_utils.h"
#include "cam_app.h"

/*
static Ecore_Timer *conti_update_progressbar_timer = NULL;
*/
static int pre_shot_num = 0;
static pthread_t file_save_thread;
static pthread_mutex_t file_save_mutex;
static pthread_cond_t file_save_cond;
static GQueue *file_save_queue;

static gboolean __data_init(void *data);
static Eina_Bool __update_pb_timer_cb(void *data);
static void __set_error(CamContinuousShotData *continuous_shot_data,
		      				int code, char *message);


static gboolean __data_init(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;
	gchar *temp_file_name1 = NULL, *filename = NULL;

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, FALSE, "camapp_handle is NULL");

	camapp->continuous_shot_data = (CamContinuousShotData *)malloc(sizeof(CamContinuousShotData));
	if (camapp->continuous_shot_data == NULL) {
		cam_critical(LOG_SYS, "continuous_shot_data malloc failed.");
		return FALSE;
	}

	camapp->continuous_shot_data->shot_num = 0;
	camapp->continuous_shot_data->lock = FALSE;
	camapp->continuous_shot_data->capture_status = CAM_CONTI_SHOT_STATUS_DEFAULT;
	camapp->continuous_shot_data->error = NULL;

	temp_file_name1 = cam_app_get_next_filename(CAM_FILE_EXTENTION_IMAGE);
	if (temp_file_name1) {
		filename = g_path_get_basename(temp_file_name1);
		if (filename) {
			char* save_string = NULL;
			char* first_filename = strtok_r(filename, ".",&save_string);
			if (first_filename == NULL) {
				if (filename != NULL) {
					free(filename);
					filename = NULL;
					save_string = NULL;
				}
				return FALSE;
			}
			camapp->continuous_shot_data->first_filename = strdup(first_filename);
			if (filename != NULL) {
				free(filename);
				filename = NULL;
				save_string = NULL;
			}
		}
		free(temp_file_name1);
		temp_file_name1 = NULL;
	}

	return TRUE;
}


static Eina_Bool __update_pb_timer_cb(void *data)
{
	struct appdata *ad = data;
	cam_retv_if(ad == NULL, FALSE);
	CamAppData *camapp = ad->camapp_handle;
	cam_retv_if(camapp == NULL, FALSE);
	char pb_text[10] = "";
	double pb_value = 0.0;
	camapp = ad->camapp_handle;
	DEBUG_TRACE("camapp->continuous_shot_data->shot_num %d", camapp->continuous_shot_data->shot_num);
	DEBUG_TRACE("pre_shot_num %d", pre_shot_num);
	if (camapp->continuous_shot_data->shot_num > pre_shot_num) {
		pb_value =
		    (double)camapp->continuous_shot_data->shot_num /
		    (double)MAX_CONTINUOUS_SHOT;
		cam_app_set_progressbar_value(ad, pb_value);

		snprintf(pb_text, sizeof(pb_text), "%d/%d",
			 camapp->continuous_shot_data->shot_num,
			 MAX_CONTINUOUS_SHOT);
		cam_app_set_progressbar_text(ad, pb_text);

		pre_shot_num = camapp->continuous_shot_data->shot_num;
	}

	return 1;
}

void cam_continuous_shot_update_progress_value(void *data)
{
	struct appdata *ad = data;
	cam_ret_if(ad == NULL);
	CamAppData *camapp = ad->camapp_handle;
	cam_ret_if(camapp == NULL);
	cam_ret_if(camapp->continuous_shot_data == NULL);

	char pb_text[10] = "";
	double pb_value = 0.0;
	camapp = ad->camapp_handle;
	DEBUG_TRACE("camapp->continuous_shot_data->shot_num %d", camapp->continuous_shot_data->shot_num);
	DEBUG_TRACE("pre_shot_num %d", pre_shot_num);
	DEBUG_TRACE("ad->progressbar %p", ad->progressbar);
	if (camapp->continuous_shot_data->shot_num > pre_shot_num) {
		pb_value = (double)camapp->continuous_shot_data->shot_num / (double)MAX_CONTINUOUS_SHOT;
		cam_app_set_progressbar_value(ad, pb_value);
		snprintf(pb_text, sizeof(pb_text), "%d/%d", camapp->continuous_shot_data->shot_num, MAX_CONTINUOUS_SHOT);
		cam_app_set_progressbar_text(ad, pb_text);
		pre_shot_num = camapp->continuous_shot_data->shot_num;
	}

}

static void __set_error(CamContinuousShotData *continuous_shot_data, int code, char *message)
{
	cam_debug(LOG_SYS, " start");

	if (continuous_shot_data == NULL)
		return;

	if (continuous_shot_data->error == NULL) {
		if (cam_utils_check_mmc_status() == VCONFKEY_SYSMAN_MMC_REMOVED) {
			continuous_shot_data->error =
			    cam_error_new_literal(CAM_CONTI_SHOT_ERROR_MMC_REMOVED,
					  dgettext(PACKAGE, "IDS_CAM_POP_MEMORY_CARD_REMOVED"));
		} else {
			if (message)
				continuous_shot_data->error = cam_error_new_literal(code, message);
		}
	}

	continuous_shot_data->capture_status = CAM_CONTI_SHOT_STATUS_STOPPING;
	cam_mm_set_conti_shot_break(TRUE);

	if (continuous_shot_data->error && continuous_shot_data->error->message)
		cam_critical(LOG_SYS, " Done: Continuous shot failed [%s]", continuous_shot_data->error->message);
}


void cam_continuous_shot_data_free(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	CamAppData *camapp = NULL;

	cam_retm_if(ad == NULL, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");

	cam_debug(LOG_FILE, "");

	if (camapp->continuous_shot_data) {
		int i = 0;
		for (i = 0; i < camapp->continuous_shot_data->shot_num; i++) {
			if (camapp->continuous_shot_data->file_path[i]) {
				free(camapp->continuous_shot_data->file_path[i]);
				camapp->continuous_shot_data->file_path[i] = NULL;
			}
		}

		if(camapp->continuous_shot_data->first_filename){
			free(camapp->continuous_shot_data->first_filename);
			camapp->continuous_shot_data->first_filename = NULL;
		}

		if (camapp->continuous_shot_data->error) {
			g_error_free(camapp->continuous_shot_data->error);
			camapp->continuous_shot_data->error = NULL;
		}

		free(camapp->continuous_shot_data);
		camapp->continuous_shot_data = NULL;

		cam_debug(LOG_UI, "remove continuous_shot_data");
	}

	cam_debug(LOG_FILE, " done.");

	return;
}

void cam_continuous_shot_capture_start(void *data)
{
	DEBUG_TRACE("");
	struct appdata *ad = (struct appdata *)data;

	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;

	cam_continuous_shot_data_free(ad);
	__data_init(ad);

	DEL_EVAS_OBJECT(ad->progressbar_edje);
	elm_object_part_content_unset(ad->ug_base, "progressbar_layout");

	cam_app_create_progressbar(ad);
	ad->progressbar_edje = cam_app_load_edj(ad->ug_base, CAM_MAIN_LAYOUT_EDJ_NAME, "progressbar");

	cam_retm_if(ad->progressbar_edje == NULL,"ad->progressbar_edje is NULL");
	elm_object_part_content_set(ad->progressbar_edje, "progressbar", ad->progressbar);
	elm_object_part_content_set(ad->ug_base, "progressbar_layout", ad->progressbar_edje);

	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progressbar_edje), "progressbar,default", "prog");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progressbar_edje), "progressbar,vertical", "prog");
		break;
	default:
		DEBUG_TRACE("wow, reached un-able reached codes");
	}

	pre_shot_num = 0;

	cam_set_orient_value(ad);

	int remain = 0;
	int max_count = 0;
	remain = cam_system_get_still_count_by_resolution(ad);

	if(remain <= MAX_CONTINUOUS_SHOT)
	  	max_count = remain;
	else
		max_count = MAX_CONTINUOUS_SHOT;

	if(max_count > 1) {
		camapp->continuous_shot_data->capture_status = CAM_CONTI_SHOT_STATUS_CAPTURING;
		cam_toolbar_update(ad);
		cam_mm_continuous_capture_start(max_count, CONTINUOUS_SHOT_INTERVAL, (camera_capturing_cb)cam_continuous_shot_capture_callback, camapp->capture_completed_cb, (void*)ad);
	} else {
		cam_mm_capture_start(camapp->capture_cb, camapp->capture_completed_cb, (void*)ad);
	}
}

void *cam_continuous_shot_file_save_thread_run(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retvm_if(camapp == NULL, NULL, "camapp_handle is NULL");

	gboolean Exit_loop = FALSE;

	cam_debug(LOG_FILE, "");

	pthread_mutex_lock(&file_save_mutex);

	while (!Exit_loop) {
		cam_debug(LOG_FILE, " wait signal...");

		pthread_cond_wait(&file_save_cond, &file_save_mutex);

		cam_debug(LOG_FILE, " signal received");

		while (!g_queue_is_empty(file_save_queue)) {
			File_Save_Info *file_info = NULL;
			/*TODO: consider sync*/
			file_info = g_queue_pop_head(file_save_queue);

			if ((file_info == NULL) || (file_info->cmd == NULL)) {
				__set_error(camapp->continuous_shot_data, CAM_CONTI_SHOT_ERROR_FILE_WRITE, _("File Info Error"));
				pthread_mutex_unlock(&file_save_mutex);
				return NULL;
			}

			DEBUG_TRACE("%p", file_info);
            DEBUG_TRACE("file_info->cmd %s", file_info->cmd);
			DEBUG_TRACE("file_info->filename %s", file_info->filename);

			if (strcmp(SAVE_THREAD_EXIT, file_info->cmd) == 0) {
				if (file_info->buffer != NULL) {
					free(file_info->buffer);
					file_info->buffer = NULL;
				}
				if (file_info->filename != NULL) {
					free(file_info->filename);
					file_info->filename = NULL;
				}
				if (file_info->cmd == NULL) {
					free(file_info->cmd);
					file_info->cmd = NULL;
				}
				free(file_info);
				file_info = NULL;
				Exit_loop = TRUE;
				break;
			}

			if (strcmp(SAVE_THREAD_FINISH, file_info->cmd) == 0) {
			 	sync();
				cam_utils_request_main_pipe_handler(ad, NULL, CAM_MAIN_PIPE_OP_TYPE_BEST_SHOT_IMAGE_HANDLE);
	        } else {
	        	if (file_info->filename == NULL) {
					__set_error(camapp->continuous_shot_data, CAM_CONTI_SHOT_ERROR_FILE_WRITE, _("File Name Error"));
					pthread_mutex_unlock(&file_save_mutex);
					return NULL;
        		}

				/*note: write file*/
				FILE *fd = fopen(file_info->filename, "w+");
				if (fd == NULL) {
					__set_error(camapp->continuous_shot_data, CAM_CONTI_SHOT_ERROR_FILE_WRITE, _("File Open Fail"));
					pthread_mutex_unlock(&file_save_mutex);
					return NULL;
				} else {
					int ret = fwrite(file_info->buffer, file_info->buf_size, 1, fd);
                    fclose(fd);

					fd = NULL;
					if (ret != 1) {
						__set_error(camapp->continuous_shot_data, CAM_CONTI_SHOT_ERROR_FILE_WRITE, _("File Write Fail"));
						pthread_mutex_unlock(&file_save_mutex);
						return NULL;
					}
				}

	            /*note: reg to media info*/
	           	__cam_single_shot_reg_file(file_info->filename);

				cam_debug(LOG_FILE, " register done.");

				if (file_info->buffer != NULL) {
					free(file_info->buffer);
					file_info->buffer = NULL;
				}

				if (file_info->filename != NULL) {
					free(file_info->filename);
					file_info->filename = NULL;
				}

				if (file_info->cmd == NULL) {
					free(file_info->cmd);
					file_info->cmd = NULL;
				}

				free(file_info);
				file_info = NULL;
				cam_debug(LOG_FILE, " save file done.");
			}
		}
	}

	pthread_mutex_unlock(&file_save_mutex);

	pthread_mutex_destroy(&file_save_mutex);
	pthread_cond_destroy(&file_save_cond);

	cam_debug(LOG_SYS, " thread exit...");

	return NULL;
}

void cam_continuous_shot_file_save_thread_exit(void *data)
{
	if (file_save_queue != NULL) {
		File_Save_Info *file_info = calloc(1, sizeof(File_Save_Info));

		if (file_info == NULL)
			return;

		file_info->buffer = NULL;
		file_info->buf_size = 0;
		file_info->filename = NULL;
		file_info->cmd = strdup(SAVE_THREAD_EXIT);
		g_queue_push_tail(file_save_queue, file_info);
		pthread_cond_signal(&file_save_cond);
	}
	return;
}

gboolean cam_continuous_shot_create_file_save_thread(void *data)
{
	struct appdata *ad = (struct appdata *)data;

	int err = 0;
	cam_debug(LOG_SYS, "");

	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	err = pthread_mutex_init(&file_save_mutex, NULL);
	if (err != 0) {
		cam_critical(LOG_CAM,
			     "Create Camera file register mutex failed");
		return FALSE;
	}

	err = pthread_cond_init(&file_save_cond, NULL);
	if (err != 0) {
		cam_critical(LOG_CAM,
			     "Create Camera file register cond failed");
		return FALSE;
	}

	file_save_queue = g_queue_new();
	if (file_save_queue == NULL) {
		cam_critical(LOG_CAM,
			     "Create Camera file register queue failed");
		return FALSE;
	}

	err = pthread_create(&file_save_thread, NULL, cam_continuous_shot_file_save_thread_run, (void *)ad);
	if (err != 0) {
		cam_critical(LOG_CAM,
			     "Create Camera file register thread failed");
		return FALSE;
	}

	return TRUE;
}

void cam_continuous_shot_finish_save_image()
{
	File_Save_Info *file_info = calloc(1, sizeof(File_Save_Info));

	if (file_info == NULL)
		return;

	file_info->buffer = NULL;
	file_info->buf_size = 0;
	file_info->filename = NULL;
	file_info->cmd = strdup(SAVE_THREAD_FINISH);

	DEBUG_TRACE("file_info->cmd %s", file_info->cmd);
	g_queue_push_tail(file_save_queue, file_info);
	pthread_cond_signal(&file_save_cond);
}

void cam_continuous_shot_capture_callback(camera_image_data_s* image,
								camera_image_data_s* postview,
								camera_image_data_s* thumbnail,
								void *user_data)
{
	struct appdata *ad = (struct appdata *)user_data;
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp is NULL");

	cam_debug(LOG_MM, "");

	if (ad->ug_state == CAM_UG_TERMINATE_STATE
		|| ad->ug_state == CAM_UG_PAUSE_STATE) {/*NOTE: in pause state, and terminate state, not cb*/
		return;
	}

	gchar *filename = NULL;

	if (image == NULL) {
		if (camapp->filename)
			free(camapp->filename);
		camapp->filename = NULL;
		DEBUG_TRACE(" ERROR - capture data is NULL");
		cam_mm_set_conti_shot_break(TRUE);
		return;
	}
	if (image->data == NULL) {
		if (camapp->filename)
			free(camapp->filename);
		camapp->filename = NULL;
		DEBUG_TRACE(" ERROR - capture data is NULL");
		cam_mm_set_conti_shot_break(TRUE);
		return;
	}


	if (camapp->continuous_shot_data == NULL) {
		cam_critical(LOG_SYS, " continuous_shot_data is NULL");
		CAM_TA_ACUM_ITEM_END(" cam_continuous_shot_capture_callback", 0);
		cam_mm_set_conti_shot_break(TRUE);
		return;
	}

	if (camapp->continuous_shot_data->lock == TRUE) {
		cam_debug(LOG_MM, " Skip this data... because capture was stopped.");
		CAM_TA_ACUM_ITEM_END(" cam_continuous_shot_capture_callback", 0);
		cam_mm_set_conti_shot_break(TRUE);
		return;
	}

	camapp->continuous_shot_data->lock = TRUE;

#ifdef USE_CAMERA_APP_SHUTTER_SOUND
	__cam_ta__(" cam_sound_play", cam_sound_play(camapp->shutter_sound, ad););
#endif
	filename = cam_file_get_next_filename_for_multishot(cam_app_get_target_path(),
								(const gchar *)camapp->continuous_shot_data->first_filename,
								(const gint) camapp->continuous_shot_data->shot_num,
								(const gchar *)".jpg");

	CAM_TA_ACUM_ITEM_BEGIN("		File open+write", 0);

	if (filename == NULL) {
		DEBUG_TRACE("");
		return;
	}

	if (image->format == CAMERA_PIXEL_FORMAT_JPEG) {
		/*TODO: use thread to save file*/
		void *image_buffer = calloc(1, image->size);
		if (image_buffer == NULL)
			return;

		memcpy(image_buffer, image->data, image->size);
		File_Save_Info *file_info = calloc(1, sizeof(File_Save_Info));

		if (file_info == NULL) {
			if (image_buffer)
				free(image_buffer);
			return;
		}

		file_info->buffer = image_buffer;
		file_info->buf_size = image->size;
		file_info->filename = strdup(filename);
		file_info->cmd = strdup("run_thread");

        DEBUG_TRACE("file_info->cmd %s", file_info->cmd);
		g_queue_push_tail(file_save_queue, file_info);
		pthread_cond_signal(&file_save_cond);

		{/*TODO: consider  synchro issue in future*/
			if (camapp->continuous_shot_data) {
				camapp->continuous_shot_data->file_path[camapp->continuous_shot_data->shot_num] = filename;
				camapp->continuous_shot_data->shot_num++;
				cam_debug(LOG_MM, " Continuous shot captured [%s] count [%d]", filename, camapp->continuous_shot_data->shot_num);
			} else {
				cam_debug(LOG_MM, " continuous_shot_data is null");
				return;
			}
		}
	}

	CAM_TA_ACUM_ITEM_END("		  File open+write", 0);

	if (camapp->continuous_shot_data->capture_status == CAM_CONTI_SHOT_STATUS_STOPPING
		&& camapp->continuous_shot_data->shot_num == 1) {
		cam_mm_set_conti_shot_break(TRUE);
	}

	camapp->continuous_shot_data->lock = FALSE;

	cam_utils_request_main_pipe_handler(ad, NULL, CAM_MAIN_PIPE_OP_TYPE_BURST_SHOT_UPDATE_PROGRESS);

	if (ad->stop_continuous_shot == TRUE
		|| (camapp->continuous_shot_data && camapp->continuous_shot_data->shot_num == MAX_CONTINUOUS_SHOT)) {

		camapp->continuous_shot_data->capture_status = CAM_CONTI_SHOT_STATUS_STOPPING;
		cam_debug(LOG_UI, " Captured count [%d]", camapp->continuous_shot_data->shot_num);

		if (ad->stop_continuous_shot)
			cam_mm_set_conti_shot_break(TRUE);

		ad->stop_continuous_shot =FALSE;
	}

	return;

}

gboolean cam_continuous_shot_reg_file(int shot_num, char *file_path[])
{
	int i = 0;
	GError *error = NULL;

	cam_debug(LOG_FILE, "");

	for (i = 0; i < shot_num; i++) {
		if (file_path[i]) {
			cam_debug(LOG_FILE, "register file : %s", file_path[i]);

			if (!cam_file_register(file_path[i], &error)) {
				cam_critical(LOG_FILE,
					     "cam_file_register fail");
				if (error != NULL) {
					cam_critical(LOG_FILE,
						     "cam_file_register error [%s]",
						     error->message);
					g_error_free(error);
					error = NULL;
				}
			}
		}
	}

	cam_debug(LOG_SYS, " file register done.");

	return TRUE;
}

