/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_debug.h"
#include <stdio.h>
#include <string.h>
#include <unistd.h>
/* #include <debug-message.h> */

#define LOG_LEVEL_FLAGS	(G_LOG_LEVEL_INFO | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_DEBUG | \
						G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | \
						G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION)

#define LOG_FILTER_CRITICAL		G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR
#define LOG_FILTER_WARNING		LOG_FILTER_CRITICAL | G_LOG_LEVEL_WARNING
#define LOG_FILTER_MESSAGE		LOG_FILTER_WARNING | G_LOG_LEVEL_MESSAGE
#define LOG_FILTER_INFO			LOG_FILTER_MESSAGE | G_LOG_LEVEL_INFO
#define LOG_FILTER_DEBUG		LOG_FILTER_INFO | G_LOG_LEVEL_DEBUG
#define LOG_FILTER_ALL			LOG_FILTER_DEBUG

#define BG_BLACK	40
#define FG_RED		31
#define FG_GREEN	32
#define FG_YELLOW	33
#define FG_PUPPLE	35
#define FG_CYAN		36
#define FG_BLUE		34

#define __cam_print_debug(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm[DEBUG.%s] %s", BG_BLACK, FG_GREEN, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#define __cam_print_info(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm[INFO.%s] %s", BG_BLACK, FG_PUPPLE, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#define __cam_print_msg(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm[MESSAGE.%s] %s", BG_BLACK, FG_CYAN, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#define __cam_print_warning(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm[WARNING.%s] %s", BG_BLACK, FG_YELLOW, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#define __cam_print_critical(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm[CRITICAL.%s] %s", BG_BLACK, FG_RED, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#define __cam_print_error(domain, msg) \
do { \
	printf("\x1b[%dm\x1b[%dm**ERROR.%s: %s", BG_BLACK, FG_RED, domain, msg); \
	printf("\x1b[0m\n"); \
} while (0)

#ifdef ENABLE_TIME_MEASURE

static GTimer *g_timer = NULL;
static gdouble previous_elapsed = 0;

static GTimer *g_measure_timer = NULL;
static gdouble previous_time[MEASURE_TYPE_MAX] = { 0 };

static char debug_type[MEASURE_TYPE_MAX][200] = {
	"NORMAL_MEASURE_TYPE",
	"APP_LAUNCHING_TYPE",
	"CAPTURE_MEASURE_TIME",
	"REGISTER_FILE_MEASURE_TIME",
	"REC_START_MEASURE_TIME",
	"REC_COMMIT_MEASURE_TIME",
	"REVIEW_MEASURE_TIME",
	"AUTO_FOCUSE_TIME",
};

#ifdef MEMCHECK

gboolean memcheck = false;

EXPORT_API void debug_start_memcheck()
{
	if (!memcheck) {
		mwInit();
		memcheck = true;
	}
}

EXPORT_API void debug_stop_memcheck()
{
	if (memcheck) {
		mwFlushNow();
		mwTerm();
	}
}
#endif

void debug_measure_init()
{
	g_measure_timer = g_timer_new();

	int type = 0;

	for (type = 0; type < MEASURE_TYPE_MAX; type++) {
		previous_time[type] = 0;
	}

	return;
}

void debug_measure_close()
{
	if (g_measure_timer) {
		g_timer_destroy(g_measure_timer);
		g_measure_timer = NULL;
	}

	int type = 0;

	for (type = 0; type < MEASURE_TYPE_MAX; type++) {
		previous_time[type] = 0;
	}

	return;
}

void debug_measure_start(cam_time_meature_type meature_type,
			 const gchar *format, ...)
{
	char buf[512] = { '\0', };
	va_list ap;
	va_start(ap, format);
	vsnprintf(buf, sizeof(buf), format, ap);
	va_end(ap);

	bool request_reset = true;
	int type = 0;
/*
	if(meature_type ==APP_LAUNCHING_TYPE)
	{
		if(g_measure_timer == NULL)
		{
			debug_measure_init();
			previous_time[meature_type] = g_timer_elapsed(g_measure_timer, NULL);
		}
	}
*/
	if (g_measure_timer == NULL)
		debug_measure_init();

	for (type = 0; type < MEASURE_TYPE_MAX; type++) {
		if (previous_time[type] != 0)
			request_reset = false;
	}

	if (request_reset)
		g_timer_reset(g_measure_timer);

	previous_time[meature_type] = g_timer_elapsed(g_measure_timer, NULL);

	/* cam_debug_time(LOG_TIME, "START %s : %s",debug_type[meature_type] , buf); */
}

void debug_measure_stop(cam_time_meature_type meature_type,
			const gchar *format, ...)
{
	char buf[512] = { '\0', };
	va_list ap;
	va_start(ap, format);
	vsnprintf(buf, sizeof(buf), format, ap);
	va_end(ap);

	gdouble current_elapsed = g_timer_elapsed(g_measure_timer, NULL);
	gdouble meature_time = 0.0;

	meature_time = (current_elapsed - previous_time[meature_type]);

	cam_debug_time(LOG_TIME, "%s measure time = %.02f (sec) function %s\n",
		       debug_type[meature_type], (gfloat) (meature_time), buf);

	previous_time[meature_type] = 0;
}

void debug_time_measure_log(const gchar *format, ...)
{
/*
	g_return_if_fail(g_timer);
	gdouble elapsed = g_timer_elapsed(g_timer, NULL);

	char buf[512] = {'\0',};
	va_list ap;
	va_start(ap, format);
	vsnprintf(buf, sizeof(buf), format, ap);
	va_end(ap);

	// cam_debug_time(LOG_TIME, "%s, measure_time = %.02f (ms)", buf, (gfloat)(elapsed - previous_elapsed));

	previous_elapsed = elapsed;
*/

}

void debug_time_measure_start(const gchar *format, ...)
{
	char buf[512] = { '\0', };
	va_list ap;
	va_start(ap, format);
	vsnprintf(buf, sizeof(buf), format, ap);
	va_end(ap);

	if (g_timer == NULL) {
		g_timer = g_timer_new();
	} else {
		g_timer_start(g_timer);
	}

	previous_elapsed = g_timer_elapsed(g_timer, NULL);

	/*
	   cam_debug_time(LOG_TIME, "\nNEW MEASUREMENT: %s", buf);
	   cam_debug_time(LOG_TIME, "UNIT NAME, TOTAL ELAPSED, ELAPSED");
	 */
}

void debug_time_measure_stop(void)
{
	if (g_timer) {
		g_timer_destroy(g_timer);
		g_timer = NULL;
	}
}
#endif				/*ENABLE_TIME_MEASURE */

#ifdef DEBUG_MESSAGE_ON
#ifndef USE_DLOG_MESSAGE
void
cam_log_func(const gchar *_domain, GLogLevelFlags log_level,
	     const char *file_name, const char *func, int line_number,
	     const char *format, ...)
{

	va_list args;
	char msg_buf[1024] = "\0", message[2048] = "\0";
	char *p = NULL;
	GET_FILE_NAME(p, (char *)file_name);

	va_start(args, format);
	vsnprintf(msg_buf, sizeof(msg_buf), format, args);

	snprintf(message, sizeof(message), "[%s:%s:#%d]%s", p, func,
		 line_number, msg_buf);
	va_end(args);

	if (log_level & G_LOG_LEVEL_DEBUG) {
		__cam_print_debug(_domain, message);
	} else if (log_level & G_LOG_LEVEL_INFO) {
		__cam_print_info(_domain, message);
	} else if (log_level & G_LOG_LEVEL_MESSAGE) {
		__cam_print_msg(_domain, message);
	} else if (log_level & G_LOG_LEVEL_WARNING) {
		__cam_print_warning(_domain, message);
	} else if (log_level & G_LOG_LEVEL_CRITICAL) {
		__cam_print_critical(_domain, message);
	} else if (log_level & G_LOG_LEVEL_ERROR) {
		__cam_print_error(_domain, message);
	}

}
#endif
#endif
