/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_popup.h"
#include "cam_debug.h"
#include "cam.h"

static GList *g_pop_list = NULL;

static double bar_percent = 0;
static char *bar_text = NULL;
static gboolean add_progress_bar = FALSE;


/* remove popup which is add by cam_app_*_popup() functions */
void cam_popup_remove(Evas_Object *popup)
{
	if (popup == NULL)
		return;
	GList *popup_list = NULL;
	popup_list = g_pop_list;
	g_pop_list = g_list_remove(popup_list, popup);
	evas_object_del(popup);
	popup = NULL;
}

void cam_popup_remove_all(void)
{
	GList *popup_list = NULL;
	popup_list = g_pop_list;

	if (popup_list) {
		while (popup_list) {
			GList *next = popup_list->next;
			evas_object_del(popup_list->data);
			popup_list->data = NULL;
			popup_list = next;
		}
		g_list_free(popup_list);
		g_pop_list = NULL;
	}
	return;
}

#ifdef USE_ELM_POPUP /*it is not use*/

/* popup with one button,  ELM_POPUP_RESPONSE_OK is sent along with response signal. */
void cam_popup_confirm_add(Evas_Object *window, const char *message,
			   const char *btn_name, cam_popup_respons_cb callback,
			   void *user_data)
{

	Evas_Object *popup = NULL;

	if (callback == NULL) {	/* response_cb should be set */
		cam_critical(LOG_UI,
			     "add info popup [%s]  failed, callback is NULL ",
			     message);
		return;
	}

	popup = elm_popup_add_with_buttons(window, NULL, message,
					   1, btn_name, ELM_POPUP_RESPONSE_OK,
					   NULL);

	if (popup == NULL) {
		cam_critical(LOG_UI, "add confirm popup [%s]  failed ",
			     message);
		return;
	}
	evas_object_smart_callback_add(popup, "response", callback, user_data);
	evas_object_show(popup);

	g_pop_list = g_list_append(g_pop_list, popup);

	return;
}

/* popup with two button, ELM_POPUP_RESPONSE_OK/ELM_POPUP_RESPONSE_CANCEL is sent along with response signal. */
void cam_popup_question_add(Evas_Object *window, const char *message,
			    const char *btn_n_1, const char *btn_n_2,
			    cam_popup_respons_cb callback, void *user_data)
{
	Evas_Object *popup = NULL;

	if (callback == NULL) {	/* response_cb should be set */
		cam_critical(LOG_UI,
			     "add info popup [%s]  failed, callback is NULL ",
			     message);
		return;
	}

	popup = elm_popup_add_with_buttons(window, NULL, message, 2,
					   btn_n_1, btn_n_2,
					   ELM_POPUP_RESPONSE_OK,
					   ELM_POPUP_RESPONSE_CANCEL, NULL);

	if (popup == NULL) {
		cam_critical(LOG_UI, "add question popup [%s]  failed ",
			     message);
		return;
	}
	evas_object_smart_callback_add(popup, "response", callback, user_data);
	evas_object_show(popup);

	g_pop_list = g_list_append(g_pop_list, popup);

	return;
}
#else				/* #ifdef USE_ELM_POPUP */
void
cam_popup_question_add(Evas_Object *window, const char *message,
		       const char *lbtn_name, void (*left_btn_cb) (void *data,
								   Evas_Object
								   *obj,
								   void
								   *event_info),
		       const char *rbtn_name, void (*right_btn_cb) (void *data,
								    Evas_Object
								    *obj,
								    void
								    *event_info))
{
	Evas_Object *notify = NULL;
	Evas_Object *layout = NULL;
	Evas_Object *label = NULL;
	Evas_Object *button = NULL;
	int width = 0;
	char buf[50] = { '\0', };

	notify = elm_notify_add(window);
	evas_object_size_hint_weight_set(notify, EVAS_HINT_EXPAND,
					 EVAS_HINT_EXPAND);
	elm_notify_align_set(notify, 0.5, 0.5);
	elm_notify_allow_events_set(notify, EINA_FALSE);
	evas_object_show(notify);

	layout = elm_layout_add(window);
	elm_layout_theme_set(layout, "standard", "notify", "style3");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND,
					 EVAS_HINT_EXPAND);
	elm_object_content_set(notify, layout);
	evas_object_show(layout);

	label = elm_label_add(window);

	elm_object_style_set(label, "popup/description");
	edje_object_part_geometry_get((Evas_Object *)
				      elm_layout_edje_get(layout),
				      "elm.swallow.content", NULL, NULL, &width,
				      NULL);
	elm_label_line_wrap_set(label, EINA_TRUE);
	elm_label_wrap_width_set(label, width);

	snprintf(buf, sizeof(buf), "<align=middle>%s", message);

	elm_object_text_set(label, buf);
	elm_object_part_content_set(layout, "elm.swallow.content", label);
	evas_object_show(label);

	if (lbtn_name && left_btn_cb) {
		button = elm_button_add(window);
		elm_object_text_set(button, lbtn_name);
		elm_object_part_content_set(layout, "elm.swallow.left", button);
		evas_object_smart_callback_add(button, "focused", left_btn_cb,
					       notify);
		evas_object_show(button);
	}

	if (rbtn_name && right_btn_cb) {
		button = elm_button_add(window);
		elm_object_text_set(button, rbtn_name);
		elm_object_part_content_set(layout, "elm.swallow.right", button);
		evas_object_smart_callback_add(button, "focused", right_btn_cb,
					       notify);
		evas_object_show(button);
	}

	g_pop_list = g_list_append(g_pop_list, notify);

	return;
}
#endif				/*  #ifdef USE_ELM_POPUP */

Evas_Object *cam_popup_guide_screen_add(Evas_Object *window, const char *message)
{
	cam_debug(LOG_UI, "");

	Evas_Object *ctxpopup = NULL;
	ctxpopup = elm_ctxpopup_add(window);

	if (!ctxpopup) {
		cam_critical(LOG_UI, "critical : can't make ctx popup");
		return NULL;
	} else {
		elm_object_style_set(ctxpopup, "beat_winset_test/imageeditor");
		elm_ctxpopup_item_append(ctxpopup, _("Flickr"), NULL, NULL,
					 ctxpopup);
		evas_object_move(ctxpopup, 200, 200);
		evas_object_show(ctxpopup);
	}

	return ctxpopup;
}


gboolean cam_popup_create(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup != NULL)
		cam_popup_delete(ad);

	elm_object_part_content_unset(ad->ug_base, "progress_popup_layout");

	// create popup
	ad->progress_popup = cam_app_load_edj(ad->ug_base, CAM_UTILS_EDJ_NAME, "progress_popup");

	cam_retvm_if(ad->progress_popup == NULL, FALSE, "progress_popup is NULL");
	elm_object_part_content_set(ad->ug_base, "progress_popup_layout", ad->progress_popup);

	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progress_popup), "progress_popup,default", "prog");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progress_popup), "progress_popup,vertical", "prog");
		break;
	default:
		DEBUG_TRACE("wow, reached un-able reached codes");
	}

	evas_object_show(ad->progress_popup);

	return TRUE;
}

// create popup with progress bar burst_progress_popup
gboolean cam_popup_for_burst_create(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup != NULL)
		cam_popup_delete(ad);

	elm_object_part_content_unset(ad->ug_base, "progress_popup_layout");

	// create popup
	ad->progress_popup = cam_app_load_edj(ad->ug_base, CAM_UTILS_EDJ_NAME, "burst_progress_popup");

	cam_retvm_if(ad->progress_popup == NULL, FALSE, "progress_popup is NULL");
	elm_object_part_content_set(ad->ug_base, "progress_popup_layout", ad->progress_popup);

	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progress_popup), "progress_popup,default", "prog");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		edje_object_signal_emit(_EDJ(ad->progress_popup), "progress_popup,vertical", "prog");
		break;
	default:
		DEBUG_TRACE("wow, reached un-able reached codes");
	}

	evas_object_show(ad->progress_popup);

	return TRUE;
}


gboolean cam_popup_add_progress_bar(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup_bar != NULL) {
		evas_object_del(ad->progress_popup_bar);
		ad->progress_popup_bar = NULL;
	}

	ad->progress_popup_bar = elm_progressbar_add(ad->ug_base);
	cam_retvm_if(ad->progress_popup_bar == NULL, FALSE, "Cannot add progressbar object\n");

	Evas_Object *pb = ad->progress_popup_bar;
	edje_object_part_swallow(_EDJ(ad->progress_popup), "progress_popup_bar", pb);
	elm_object_style_set(pb, "list_progress");
	evas_object_size_hint_align_set(pb, EVAS_HINT_FILL, EVAS_HINT_FILL);
	add_progress_bar = TRUE;

	return TRUE;
}

gboolean cam_popup_set_text(void *data, char *text)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup == NULL )
		return FALSE;

	if (text != NULL) {
		bar_text = text;
		edje_object_part_text_set(_EDJ(ad->progress_popup), "progress_popup_text", text);
	}
	return TRUE;
}

gboolean cam_popup_set_bar_value(void *data, double value)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup_bar) {
		bar_percent = value;
		elm_progressbar_value_set(ad->progress_popup_bar, value);
		evas_object_raise(ad->progress_popup_bar);
	}

	return TRUE;
}

gboolean cam_popup_delete(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retvm_if(ad == NULL, FALSE, "appdata is NULL");

	if (ad->progress_popup != NULL) {
		evas_object_del(ad->progress_popup);
		ad->progress_popup = NULL;
	}

	if (ad->progress_popup_bar != NULL) {
		evas_object_del(ad->progress_popup_bar);
		ad->progress_popup_bar = NULL;
	}

	bar_percent = 0;
	bar_text = NULL;
	add_progress_bar = FALSE;

	return TRUE;
}


void cam_popup_rotate(void *data)
{
	struct appdata *ad = (struct appdata *)data;
	cam_retm_if(ad == NULL, "appdata is NULL");

	unsigned int tmp_percent = bar_percent;
	char *tmp_text = bar_text;
	gboolean using_bar = TRUE;

	cam_popup_delete(ad);

	if (!cam_popup_create(ad)) {
		cam_critical(LOG_UI, "cam_popup_create() failed");
		return;
	}

	if (add_progress_bar) {
		if (!cam_popup_add_progress_bar(ad))
			cam_debug(LOG_UI, "cam_popup_add_progress_bar() failed");

		if (!cam_popup_set_bar_value(ad, tmp_percent))
			cam_debug(LOG_UI, "cam_popup_set_bar_value() failed");
	}

	if (!cam_popup_set_text(ad, tmp_text))
		cam_debug(LOG_UI, "cam_popup_set_text() failed");
}

