/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "cam_voice_command.h"
#include "cam_debug.h"

// voice control module is not setup.
#ifdef ENABLE_VOICE_CONTROL
#include <vc_bargein.h>

static vc_bargein_h voice_cmd_h = NULL;
static cam_voice_cmd_cb g_cmd_cb;

static void __init_cb(vc_bargein_h vc, void *user_data);
static void __result_cb(vc_bargein_h vc, vc_bargein_result_event_e event, vc_bargein_app_type_e app_type, int action_type, void *user_data);
static void __error_cb(vc_bargein_h vc, vc_bargein_error_e reason, void *user_data);
static void __start_voice_cmd(void *user_data);

static void __init_cb(vc_bargein_h vc, void *user_data)
{

	cam_retm_if(user_data == NULL, "user_data is NULL");

	if (vc == NULL) {
		cam_debug(LOG_CAM, "voice handle init fail");
		g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);
	}

	__start_voice_cmd(user_data);

	return;
}

static void __result_cb(vc_bargein_h vc, vc_bargein_result_event_e event, vc_bargein_app_type_e app_type, int action_type, void *user_data)
{
	if (voice_cmd_h == NULL) {
		cam_debug(LOG_CAM, "voice_cmd_h is NULL");
		g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);
	}

	if (app_type == VC_BARGEIN_APP_TYPE_CAMERA && action_type == VC_BARGEIN_CAMERA_ACTION_CAPTURE) {
		switch(event) {
		case VC_BARGEIN_RESULT_EVENT_SUCCESS:
			{
				g_cmd_cb(CAM_VOICE_CMD_READY, user_data);
			}
			break;
		case VC_BARGEIN_RESULT_EVENT_REJECTED:
			{
				g_cmd_cb(CAM_VOICE_CMD_REJECT, user_data);
			}
			break;
		case VC_BARGEIN_RESULT_EVENT_ERROR:
		default:
			g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);
			return;
		}
	}
	else
		g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);

	return;
}

static void __error_cb(vc_bargein_h vc, vc_bargein_error_e reason, void *user_data)
{
	cam_debug(LOG_CAM, "error is [%d]", reason);
	g_cmd_cb(CAM_VOICE_CMD_READY, user_data);
	return;
}

static void __start_voice_cmd(void *user_data)
{
	if (voice_cmd_h == NULL) {
		cam_debug(LOG_CAM, "voice_cmd_h is NULL");
		g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);
	}

	if (vc_bargein_start(voice_cmd_h, VC_BARGEIN_LANGUAGE_KO_KR, VC_BARGEIN_APP_TYPE_CAMERA) != VC_BARGEIN_ERROR_NONE) {
		cam_debug(LOG_CAM, "vc_bargein_start() fail");
		g_cmd_cb(CAM_VOICE_CMD_ERROR, user_data);
	}
	return;
}

#endif

gboolean cam_voice_cmd_create(cam_voice_cmd_cb cmd_cb, void *user_data)
{
#ifdef ENABLE_VOICE_CONTROL

	cam_retvm_if(user_data == NULL, FALSE, "user_data is NULL");
	if (voice_cmd_h) {
		vc_bargein_cancel(voice_cmd_h);
		vc_bargein_destroy(voice_cmd_h);
		voice_cmd_h = NULL;
	}

	g_cmd_cb = cmd_cb;

	if (vc_bargein_create(&voice_cmd_h, (vc_bargein_initialized_cb)__init_cb,
		(vc_bargein_result_cb)__result_cb, (vc_bargein_error_cb)__error_cb, user_data) != VC_BARGEIN_ERROR_NONE) {
		cam_debug(LOG_CAM, "vc_bargein_create() fail");
		return FALSE;
	}
#endif

	return TRUE;
}

void cam_voice_cmd_destroy()
{
#ifdef ENABLE_VOICE_CONTROL
	if (voice_cmd_h == NULL) {
		cam_debug(LOG_CAM, "voice_cmd_h is already delete");
		return;
	}

	vc_bargein_cancel(voice_cmd_h);
	vc_bargein_destroy(voice_cmd_h);
	voice_cmd_h = NULL;
#endif
	return;
}

