/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include "camera_utils.h"

#include "cam_indicator_edc_callback.h"
#include "cam_common_edc_callback.h"
#include "cam_lbs.h"
#include "edc_string.h"
#include "edc_image_name.h"

#define CHARGING_INTERVAL	(0.5)
#define SECONDS_IN_HOUR (1*60*60)
#define SECONDS_RECORDING_MAX ((99*60*60)+(99*60)+99)

static char *storage_image[CAM_STORAGE_NUM] = {
	INDICATOR_STORAGE_INTERNAL_ICON,
	INDICATOR_STORAGE_CARD_ICON,
	INDICATOR_STORAGE_INTERNAL_ICON,
};

#define CAM_BATTERY_LEVEL_MAX 8
static char *battery_image[CAM_BATTERY_LEVEL_MAX] = {
	INDICATOR_BATTERY_LEVEL_00_ICON,
	INDICATOR_BATTERY_LEVEL_01_ICON,
	INDICATOR_BATTERY_LEVEL_02_ICON,
	INDICATOR_BATTERY_LEVEL_03_ICON,
	INDICATOR_BATTERY_LEVEL_04_ICON,
	INDICATOR_BATTERY_LEVEL_05_ICON,
	INDICATOR_BATTERY_FULL_ICON,
	INDICATOR_BATTERY_CHARGE_ICON,
};

static char *gps_level_image[CAM_LBS_STATE_NUM] = {
	INDICATOR_GPS_STATE_OFF_ICON,
	INDICATOR_GPS_STATE_ON_ICON,
	INDICATOR_GPS_STATE_CONNECT_ANI01_ICON,
	INDICATOR_GPS_STATE_CONNECT_ANI02_ICON,
	INDICATOR_GPS_STATE_CONNECT_ANI03_ICON,
};

static char *mute_image = INDICATOR_MUTE_ICON;

static Evas_Object *indicator_image_obj_battery = NULL;
static Evas_Object *indicator_image_obj_storage = NULL;
static Evas_Object *indicator_image_obj_gps_level = NULL;
static Evas_Object *indicator_image_obj_mute = NULL;


void indicator_update_storage(struct appdata *ad)
{
	int mmc_state = -1;
	debug_fenter(LOG_UI);
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");
	const char *image_file = NULL;

	vconf_get_int(VCONFKEY_SETAPPL_DEFAULT_MEM_CAMERA_INT, &(ad->camapp_handle->storage));

	if (ad->camapp_handle->storage == CAM_STORAGE_EXTERNAL) {
		if (!vconf_get_int(VCONFKEY_SYSMAN_MMC_STATUS, &mmc_state)) {
			if (mmc_state == VCONFKEY_SYSMAN_MMC_REMOVED) {
				cam_info(LOG_UI, "MMC card is removed");
				GValue value = { 0 };
				CAM_GVALUE_SET_INT(value, CAM_STORAGE_INTERNAL);
				cam_handle_value_set(ad, PROP_STORAGE, &value);
			}
		}
	}

	switch (ad->camapp_handle->storage) {
	case CAM_STORAGE_INTERNAL:
		image_file = storage_image[CAM_STORAGE_INTERNAL];
		break;
	case CAM_STORAGE_EXTERNAL:
		image_file = storage_image[CAM_STORAGE_EXTERNAL];
		break;
	default:
		break;
	}

	if (!indicator_image_obj_storage) {
		indicator_image_obj_storage = elm_image_add(ad->indicator_edje);
	}

	cam_retm_if(indicator_image_obj_storage == NULL,
		    "indicator_image_obj_storage is NULL");
	elm_image_file_set(indicator_image_obj_storage, CAM_IMAGE_EDJ_NAME, image_file);

	if (ad->camapp_handle->camera_mode == CAM_CAMCORDER_MODE
		&& ad->is_recording == TRUE)
		edje_object_part_swallow(_EDJ(ad->indicator_edje), INDI_REC_STORAGE,
				 indicator_image_obj_storage);
	else
		edje_object_part_swallow(_EDJ(ad->indicator_edje), INDICATOR_STORAGE,
				 indicator_image_obj_storage);

	ad->remained_count = cam_system_get_still_count_by_resolution(ad);
	indicator_update_remain_count(ad);
}

void indicator_update_gps_level(struct appdata *ad)
{
	debug_fenter(LOG_UI);
	cam_retm_if(ad == NULL, "appdata is NULL");
	CamAppData *camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp is NULL");
	cam_ret_if(ad->toolbar_hided);
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");
	const char *image_file = NULL;
	/*DEBUG_TRACE("====level : %d", ad->gps_level);*/
	switch (camapp->gps_level) {
	case CAM_LBS_STATE_DISABLE:
		image_file = gps_level_image[CAM_LBS_STATE_DISABLE];
		break;
	case CAM_LBS_STATE_SERVICE_START:
		image_file = gps_level_image[CAM_LBS_STATE_SERVICE_START];
		break;
	case CAM_LBS_STATE_SERVICE_ENABLE:
		image_file = gps_level_image[CAM_LBS_STATE_SERVICE_ENABLE];
		break;
	case CAM_LBS_STATE_ACCURANCY_ROUGH:
		image_file = gps_level_image[CAM_LBS_STATE_ACCURANCY_ROUGH];
		break;
	case CAM_LBS_STATE_ACCURANCY_DETAILED:
		image_file = gps_level_image[CAM_LBS_STATE_ACCURANCY_DETAILED];
		break;
	default:
		image_file = gps_level_image[CAM_LBS_STATE_DISABLE];
	}
	if (!indicator_image_obj_gps_level) {
		indicator_image_obj_gps_level =
		    elm_image_add(ad->indicator_edje);
	}
	cam_retm_if(indicator_image_obj_gps_level == NULL,
		    "indicator_image_obj_gps_level is NULL");
	elm_image_file_set(indicator_image_obj_gps_level, CAM_IMAGE_EDJ_NAME, image_file);

	cam_debug(LOG_CAM, "ad->camapp_handle->camera_mode %d", camapp->camera_mode);
	cam_debug(LOG_CAM, "ad->mm_state %d", ad->mm_state);

	if (camapp->camera_mode == CAM_CAMCORDER_MODE
		&& ad->is_recording == TRUE) {
		if (camapp->gps_level == CAM_LBS_STATE_DISABLE) {
			HIDE_EVAS_OBJECT(indicator_image_obj_gps_level);
			edje_object_part_unswallow(_EDJ(ad->indicator_edje), indicator_image_obj_gps_level);

		} else {
			SHOW_EVAS_OBJECT(indicator_image_obj_gps_level);
			edje_object_part_swallow(_EDJ(ad->indicator_edje),
						INDI_REC_GPS,
						indicator_image_obj_gps_level);
		}
	} else {
		if (camapp->gps_level == CAM_LBS_STATE_DISABLE) {
			HIDE_EVAS_OBJECT(indicator_image_obj_gps_level);
			edje_object_part_unswallow(_EDJ(ad->indicator_edje), indicator_image_obj_gps_level);
		} else {
			SHOW_EVAS_OBJECT(indicator_image_obj_gps_level);
			edje_object_part_swallow(_EDJ(ad->indicator_edje),
						INDICATOR_GPS,
						indicator_image_obj_gps_level);
		}
	}
}

void indicator_update_gps(struct appdata *ad)
{
	debug_fenter(LOG_UI);
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");
	indicator_update_gps_level(ad);
}

void indicator_update_remain_count(struct appdata *ad)
{
	char remain[80] = "";
	debug_fenter(LOG_UI);
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");

	gint64 remained_count = 0;
	remained_count = ad->remained_count;

	if (ad->camapp_handle->camera_mode == CAM_CAMCORDER_MODE && ad->is_recording == TRUE) {
		snprintf(remain, sizeof(remain), "%s", " ");
	} else {
		if (remained_count >= 0 && remained_count < 301) {
			snprintf(remain, sizeof(remain), "%d", (int)remained_count);
		} else {
			snprintf(remain, sizeof(remain), "%d", (int)remained_count);
			snprintf(remain, sizeof(remain), "%s", " ");
		}
	}

	DEBUG_TRACE("INDICATOR REMAIN = %s ", remain);

	edje_object_part_text_set(_EDJ(ad->indicator_edje), INDICATOR_NUM, remain);
}

static Eina_Bool cam_indicator_battery_ani_cb(void *data)
{
	struct appdata *ad = data;
	CamAppData *camapp = NULL;
	cam_retvm_if(ad == NULL, 0, "appdata is NULL");
	camapp = ad->camapp_handle;
        if (camapp == NULL) {
                ad->battery_animation_timer = NULL;
                return ECORE_CALLBACK_CANCEL;

        }

	if (indicator_image_obj_battery) {
		if (evas_object_visible_get(indicator_image_obj_battery))
			evas_object_hide(indicator_image_obj_battery);
		else
			evas_object_show(indicator_image_obj_battery);
	}

	return ECORE_CALLBACK_RENEW;
}

static void cam_indicator_draw_battery_ani(void *data)
{
	struct appdata *ad = data;
	CamAppData *camapp = NULL;
	debug_fenter(LOG_UI);
	cam_retm_if(ad == NULL, "appdata is NULL");
	camapp = ad->camapp_handle;
	cam_retm_if(camapp == NULL, "camapp_handle is NULL");
	if (ad->battery_animation_timer) {
		cam_debug(LOG_UI, "battery_ani timer already exists! ");
		return;
	}
	ad->battery_animation_timer =
	    ecore_timer_add(CHARGING_INTERVAL, cam_indicator_battery_ani_cb, ad);
}

void indicator_update_battery(struct appdata *ad, int level)
{
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");
	const char *image_file = NULL;
	int battery_level = 0;
	gboolean show_icon = TRUE;

	if (ad->battery_animation_timer) {
	    ecore_timer_del(ad->battery_animation_timer);
		ad->battery_animation_timer = NULL;
	}

	if (ad->camapp_handle->battery_charging) {
		battery_level = 7;
	} else {
		if (level > 15 && level <= 20) // step2
			battery_level = 2;
		else if (level > 5 && level <= 15) // step1
			battery_level = 1;
		else if (level < 5) {// blinking
			battery_level = 0;
			cam_indicator_draw_battery_ani((void *)ad);
		}
		else
			show_icon = FALSE;
	}

	cam_debug(LOG_CAM, "level [%d] img_index[%d] show state [%d]", level, battery_level, show_icon);

	if (show_icon) {
		image_file = battery_image[battery_level];

		if (!indicator_image_obj_battery) {
			indicator_image_obj_battery = elm_image_add(ad->indicator_edje);
		}
		cam_retm_if(indicator_image_obj_battery == NULL, "indicator_image_obj_battery is NULL");
		elm_image_file_set(indicator_image_obj_battery, CAM_IMAGE_EDJ_NAME, image_file);
		if (ad->camapp_handle->camera_mode == CAM_CAMCORDER_MODE
			&& ad->is_recording == TRUE)
			edje_object_part_swallow(_EDJ(ad->indicator_edje), INDI_REC_BATTERY, indicator_image_obj_battery);
		else
			edje_object_part_swallow(_EDJ(ad->indicator_edje), INDICATOR_BATTERY, indicator_image_obj_battery);
	} else {
		if (indicator_image_obj_battery)
			evas_object_hide(indicator_image_obj_battery);
	}
}

void indicator_update_mute_icon(struct appdata *ad)
{
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	cam_retm_if(ad->indicator_edje == NULL, "indicator_edje is NULL");
	const char *image_file = NULL;

	if ((ad->camapp_handle->camera_mode == CAM_CAMCORDER_MODE)
			&& (ad->camapp_handle->recording_mode == CAM_RECORD_SLOW
				 || ad->camapp_handle->recording_mode == CAM_RECORD_FAST)) {
		if (ad->is_recording == TRUE) {
			image_file = mute_image;

			if (!indicator_image_obj_mute) {
				indicator_image_obj_mute = elm_image_add(ad->indicator_edje);
			}

			cam_retm_if(indicator_image_obj_mute == NULL, "indicator_image_obj_mute is NULL");
			elm_image_file_set(indicator_image_obj_mute, CAM_IMAGE_EDJ_NAME, image_file);
			edje_object_part_swallow(_EDJ(ad->indicator_edje), INDI_REC_MUTE, indicator_image_obj_mute);
		} else {
			DEL_EVAS_OBJECT(indicator_image_obj_mute);
		}
	} else {
		DEL_EVAS_OBJECT(indicator_image_obj_mute);
	}
}

void cam_indicator_update(struct appdata *ad)
{
#ifndef	CAMERA_MACHINE_I686
	cam_retm_if(ad == NULL, "appdata is NULL");
	cam_retm_if(ad->camapp_handle == NULL, "camapp_handle is NULL");
	indicator_update_battery(ad, ad->camapp_handle->battery_level);
	DEBUG_TRACE("charging = %d",ad->camapp_handle->battery_charging);
	indicator_update_storage(ad);
	indicator_update_gps(ad);
	indicator_update_mute_icon(ad);
#endif
	return;
}

void cam_indicator_set_show_state(struct appdata *ad, bool show)
{
	cam_retm_if(ad == NULL, "appdata is NULL");

	if (ad->indicator_edje) {
		if (show)
			evas_object_show(ad->indicator_edje);
		else
			evas_object_hide(ad->indicator_edje);
	}
}

int cam_indicator_create(struct appdata *ad)
{
	debug_fenter(LOG_UI);
	cam_retvm_if(ad == NULL, EXIT_FAILURE, "appdata is NULL");

	if (ad->indicator_edje != NULL) {
		cam_indicator_update(ad);
		evas_object_show(ad->indicator_edje);
		return EXIT_SUCCESS;
	}

	elm_object_part_content_unset(ad->ug_base, "indicator_layout");

	switch (ad->target_direction) {
	case CAM_TARGET_DIRECTION_LANDSCAPE:
		ad->indicator_edje = cam_app_load_edj(ad->ug_base, CAM_INDICATOR_EDJ_NAME, "indicator_landscape");
		break;
	case CAM_TARGET_DIRECTION_LANDSCAPE_INVERSE:
		ad->indicator_edje = cam_app_load_edj(ad->ug_base, CAM_INDICATOR_EDJ_NAME, "indicator_landscape_inverse");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT:
		ad->indicator_edje = cam_app_load_edj(ad->ug_base, CAM_INDICATOR_EDJ_NAME, "indicator_portrait");
		break;
	case CAM_TARGET_DIRECTION_PORTRAIT_INVERSE:
		ad->indicator_edje = cam_app_load_edj(ad->ug_base, CAM_INDICATOR_EDJ_NAME, "indicator_portrait_inverse");
		break;
	default:
		DEBUG_TRACE("note:REACHED UN-ABLE REACHED CODES");
	}

	cam_retvm_if(ad->indicator_edje == NULL, EXIT_FAILURE, "indicator edje load fail !!!");
	elm_object_part_content_set(ad->ug_base, "indicator_layout", ad->indicator_edje);
	cam_indicator_update(ad);

	return EXIT_SUCCESS;
}

int cam_indicator_destory(struct appdata *ad)
{
	cam_retv_if(ad == NULL, -1);

	DEL_EVAS_OBJECT(indicator_image_obj_battery);
	DEL_EVAS_OBJECT(indicator_image_obj_storage);
	DEL_EVAS_OBJECT(indicator_image_obj_gps_level);
	DEL_EVAS_OBJECT(indicator_image_obj_mute);

	DEL_EVAS_OBJECT(ad->indicator_edje);
	return EXIT_SUCCESS;
}

int cam_indicator_rotate(struct appdata *ad)
{
	cam_retv_if(ad == NULL, -1);
	if (ad->indicator_edje) {
		/* delete indicator */
		evas_object_hide(ad->indicator_edje);
		cam_indicator_destory(ad);
		cam_indicator_create(ad);
		if (ad->indicator_edje) {
			indicator_toggle(ad, FALSE);
			evas_object_show(ad->indicator_edje);
		}
	} else {
		return -1;
	}
	return 0;
}

void indicator_toggle(struct appdata *ad, gboolean hide)
{
	cam_ret_if(ad == NULL || ad->indicator_edje == NULL);

	if (!hide)
		indicator_update_gps(ad);
}
